require 'test_helper'

class V1::CollectionsControllerTest < ActionController::TestCase
  tests V1::CollectionsController
  include Devise::Test::ControllerHelpers

  test 'should get index' do
    skip('Unsolved issue: ArgumentError: wrong number of arguments (2 for 0)')
  end

 test 'should user post collection to create and return :created code' do
      auth_request users(:jack)
      post :create, params: { collection: { name: 'my collection',
                                  description: 'testing collection',
                                  owner_type: 'User', owner_id: users(:jack).id,
                                  items: [{id: learning_objects(:user_lo).id, type: "LearningObject"}],
                                  tags: [{"name":"teste"}, {"name": "tag de teste"}],
                                  subjects: [{id: subjects(:subject_one).id}],
                                  educational_stages: [{id: educational_stages(:stage_teste).id}]}
                            }
      assert_response :created
  end

  test 'should institution post collection to create and return :created code' do
    auth_request users(:jack)
    post :create, params: { collection: { name: 'my collection',
                               description: 'testing collection',
                               owner_type: 'Institution', owner_id: institutions(:ufpr).id,
                               items: [{id: learning_objects(:user_lo).id, type: "LearningObject"}],
                               subjects: [{id: subjects(:subject_one).id}],
                                 tags: [{"name":"teste"}, {"name": "tag de teste"}],
                               educational_stages: [{id: educational_stages(:stage_teste).id}]}
                          }
    assert_response :created
  end

  test 'should user post collection to create and return :unauthorized code' do
    post :create, params: { collection: {name: 'my collection',
                               description: 'testing collection',
                               owner_type: 'Institution', owner_id: institutions(:ufpr).id }
                             }
    assert_response :unauthorized
  end

  test 'should user add items to collection and return :ok code' do
    auth_request users(:john)
    post :add_object, params: { id: collections(:ufpr).id, collection: {items: [{ id: learning_objects(:lo_metadata2).id, type: 'LearningObject'}] }}

    assert_response :ok
  end

  test 'should user remove items to collection and return :ok code' do
    auth_request users(:john)
    post :add_object, params: { id: collections(:ufpr).id, collection: { items: [{ id: learning_objects(:lo_metadata).id, type: 'LearningObject'}] }}
    assert_response :ok
  end

  test 'should user add items to collection and return :unprocessable_entity code' do
    auth_request users(:john)
    post :add_object, params: { id: collections(:ufpr).id, collection: {items: [nil] }}
    assert_response :unprocessable_entity
  end

  test 'should user remove items to collection and return :unprocessable_entity code' do
    auth_request users(:john)
    post :delete_object, params: { id: collections(:ufpr).id, collection: { items: [nil] } }
    assert_response :unprocessable_entity
  end


  test 'should user tagging a collection' do
    auth_request users(:jack)
    tag = tags(:tag_three).name

    post :tagging, params: { id: collections(:ufpr).id, collection: {tag: { name: tag } }}
    assert_response :created
  end


  test 'should user untagging a collection' do
    auth_request users(:john)
    tag = tags(:tag_one).name

    post :untagging, params: { id: collections(:ufpr).id, collection:{ tags: { name: tag } }}
    assert_response :ok
  end


  test 'should institution tagging a collection' do
    auth_request users(:jack)
    institution = institutions(:ufpr)
    institution.users << users(:jack)
    tag = tags(:tag_one).name

    post :tagging, params: { id: collections(:ufpr).id, collection: { tags: { name: tag } }}
    assert_response :created
  end

  test 'should institution untagging a collection' do
    auth_request users(:john)
    institution = institutions(:ufpr)
    institution.users << users(:john)
    tag = tags(:tag_two).name

    delete :untagging, params: { id: collections(:ufpr).id, collection: { tags: { name: tag } }}
    assert_response :ok
  end


  private

  def ensure_collection_is_persisted!(collection)
    collection.save!
  end
end
